package com.ningdatech.pmapi.meeting.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingExpert;
import com.ningdatech.pmapi.meeting.entity.dto.CountConfirmByMeetingIdDTO;
import com.ningdatech.pmapi.meeting.entity.dto.MeetingAndAttendStatusDTO;
import com.ningdatech.pmapi.meeting.entity.dto.ReviewProjectDTO;
import com.ningdatech.pmapi.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.ningdatech.pmapi.meeting.entity.enumeration.ExpertInviteTypeEnum;
import com.ningdatech.pmapi.meeting.entity.req.ReviewProjectListReq;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 事务专家表 服务类
 * </p>
 *
 * @author WendyYang
 * @since 2022-07-27
 */
public interface IMeetingExpertService extends IService<MeetingExpert> {

    /**
     * 查询专家的参与状态
     *
     * @param expertId   专家ID
     * @param status     状态
     * @param meetingIds 会议ID
     * @return 会议参加状态统计
     * @author WendyYang
     **/
    List<MeetingAndAttendStatusDTO> listByExpertIdAndStatus(Long expertId, Integer status, List<Long> meetingIds);

    /**
     * 查询每个会议的确认进度
     *
     * @param meetingIds 事务ID
     * @return 确认进度
     * @author WendyYang
     **/
    Map<Long, CountConfirmByMeetingIdDTO> countConfirmedByMeetingIds(List<Long> meetingIds);

    /**
     * 根据事务ID查询所有邀请记录
     *
     * @param meetingId 事务ID
     * @return 邀请专家记录
     */
    List<MeetingExpert> listByMeetingId(Long meetingId);

    /**
     * 根据事务ID查询所有邀请记录
     *
     * @param meetingIds 事务ID
     * @return List<SubjectMatterExpert>
     */
    List<MeetingExpert> listByMeetingIds(List<Long> meetingIds);

    /**
     * 获取专家参与事务的最新记录
     *
     * @param meetingId 会议ID
     * @param expertId  专家ID
     * @return 专家参会记录
     * @author WendyYang
     **/
    MeetingExpert getByMeetingIdAndExpertId(Long meetingId, Long expertId);

    /**
     * 根据状态与会议ID分页查询专家列表
     *
     * @param page       分页
     * @param meetingId  会议ID
     * @param status     状态
     * @param inviteType 邀请类型
     * @return Page<MeetingExpert>
     * @author WendyYang
     **/
    Page<MeetingExpert> pageExpertByStatusAndMeetingId(Page<MeetingExpert> page, Long meetingId, ExpertAttendStatusEnum status, Integer inviteType);

    /**
     * 批量查询某个状态的专家邀请记录
     *
     * @param page       分页参数
     * @param meetingIds 会议ID
     * @param status     状态
     * @return 专家邀请记录
     * @author WendyYang
     **/
    Page<MeetingExpert> pageExpertByStatusAndMeetingIds(Page<MeetingExpert> page, List<Long> meetingIds, ExpertAttendStatusEnum status);

    /**
     * 根据邀请类型统计会议下某个状态的专家数量
     *
     * @param status     状态
     * @param meetingId  会议ID
     * @param inviteType 邀请类型
     * @return int
     * @author WendyYang
     **/
    int countExpertByStatusAndMeetingId(ExpertAttendStatusEnum status, Long meetingId, ExpertInviteTypeEnum inviteType);

    /**
     * 查询所有同意参加的专家记录（批量会议）
     *
     * @param meetingIds 会议ID
     * @return 同意参加的专家
     * @author WendyYang
     **/
    List<MeetingExpert> listAgreedExperts(Collection<Long> meetingIds);

    /**
     * 查询所有同意参加的专家记录（单个会议）
     *
     * @param meetingId 会议ID
     * @return 同意参加的专家
     * @author WendyYang
     **/
    default List<MeetingExpert> listAgreedExperts(Long meetingId) {
        return listAgreedExperts(Collections.singletonList(meetingId));
    }

    /**
     * 查询会议的所有被抽取人最后一条记录
     *
     * @param meetingIds 会议ID
     * @return 抽取记录
     * @author WendyYang
     **/
    List<MeetingExpert> listExpertLastByMeetingIds(Collection<Long> meetingIds);

    /**
     * 查询会议的所有被抽取人最后一条记录
     *
     * @param meetingId 会议ID
     * @return 抽取记录
     * @author WendyYang
     **/
    default List<MeetingExpert> listExpertLastByMeetingId(Long meetingId) {
        return listExpertLastByMeetingIds(Collections.singletonList(meetingId));
    }

    /**
     * 查询专家待评审项目
     *
     * @param req {@link ReviewProjectListReq}
     * @return 待评审项目分页数据
     * @author WendyYang
     **/
    Page<ReviewProjectDTO> pageReviewProjectList(ReviewProjectListReq req);

}